@@ -7,8 +7,7 @@ from django.conf import settings
 from django.core.exceptions import SuspiciousOperation
 from django.core.urlresolvers import reverse
 from django.db.models import Q, ForeignKey
-from django.db import transaction
-from django.db import IntegrityError
+from django.db import transaction, IntegrityError
 from django.core.exceptions import ObjectDoesNotExist
 from django.forms.models import inlineformset_factory
 from django.http import HttpResponseRedirect
@@ -19,13 +18,24 @@ from django.db.models.fields import AutoField, BooleanField
 from django.utils.encoding import smart_text
 
 from simple_import.compat import User
-from simple_import.models import ImportLog, ImportSetting, ColumnMatch, ImportedObject, RelationalMatch
+from simple_import.models import (ImportLog, ImportSetting, ColumnMatch,
+                                  ImportedObject, RelationalMatch)
 from simple_import.forms import ImportForm, MatchForm, MatchRelationForm
 
 
-if sys.version_info >= (3,0):
+if sys.version_info >= (3, 0):
     unicode = str
 
+
+def is_foreign_key_id_name(field_name, field_object):
+    """ Determines if field name is a ForeignKey
+    ending in "_id"
+    Used to find FK duplicates in _meta.get_all_field_names
+    """
+    if field_name[-3:] == "_id" and isinstance(field_object, ForeignKey):
+        return True
+
+
 def validate_match_columns(import_log, model_class, header_row):
     """ Perform some basic pre import validation to make sure it's
     even possible the import can work
@@ -36,11 +46,14 @@ def validate_match_columns(import_log, model_class, header_row):
     field_names = model_class._meta.get_all_field_names()
     for field_name in field_names:
         field_object, model, direct, m2m = model_class._meta.get_field_by_name(field_name)
-        # Skip if update only and skip ptr which suggests it's a django inherited field
-        # Also some hard coded ones for Django Auth
-        if import_log.import_type != "O" and field_name[-3:] != "ptr" and \
-            not field_name in ['password', 'date_joined', 'last_login']:
-            if (direct and model and not field_object.blank) or (not getattr(field_object, "blank", True)):
+        # Skip if update only and skip ptr which suggests it's a django
+        # inherited field. Also some hard coded ones for Django Auth
+        if (import_log.import_type != "O" and
+                field_name[-3:] != "ptr" and
+                field_name not in ['password', 'date_joined', 'last_login'] and
+                not is_foreign_key_id_name(field_name, field_object)):
+            if ((direct and model and not field_object.blank) or
+                    (not getattr(field_object, "blank", True))):
                 field_matches = column_matches.filter(field_name=field_name)
                 match_in_header = False
                 if field_matches:
@@ -60,7 +73,9 @@ def get_custom_fields_from_model(model_class):
     if 'custom_field' in settings.INSTALLED_APPS:
         from custom_field.models import CustomField
         try:
-            content_type = ContentType.objects.get(model=model_class._meta.module_name,app_label=model_class._meta.app_label)
+            content_type = ContentType.objects.get(
+                model=model_class._meta.module_name,
+                app_label=model_class._meta.app_label)
         except ContentType.DoesNotExist:
             content_type = None
         custom_fields = CustomField.objects.filter(content_type=content_type)
@@ -148,7 +163,7 @@ def match_columns(request, import_log_id):
         else:
             field_verbose = field_name
 
-        if direct and  not field_object.blank:
+        if direct and not field_object.blank:
             field_verbose += " (Required)"
         if direct and field_object.unique:
             field_verbose += " (Unique)"
@@ -156,6 +171,8 @@ def match_columns(request, import_log_id):
             field_verbose += " (Related)"
         elif not direct:
             add = False
+        if is_foreign_key_id_name(field_name, field_object):
+            add = False
 
         if add:
             field_choices += ((field_name, field_verbose),)
@@ -351,75 +368,77 @@ def do_import(request, import_log_id):
         if key_column_name != None and key_column_name.lower() == cell.lower():
             key_index = i
 
-    with transaction.commit_manually():
+    with transaction.atomic():
+        sid = transaction.savepoint()
         for row in import_data:
             try:
-                is_created = True
-                if import_log.import_type == "N":
-                    new_object = model_class()
-                elif import_log.import_type == "O":
-                    filters = {key_field_name: row[key_index]}
-                    new_object = model_class.objects.get(**filters)
-                    is_created = False
-                elif import_log.import_type == "U":
-                    filters = {key_field_name: row[key_index]}
-                    try:
+                with transaction.atomic():
+                    is_created = True
+                    if import_log.import_type == "N":
+                        new_object = model_class()
+                    elif import_log.import_type == "O":
+                        filters = {key_field_name: row[key_index]}
                         new_object = model_class.objects.get(**filters)
                         is_created = False
-                    except model_class.DoesNotExist:
-                        new_object = model_class()
-                new_object.simple_import_m2ms = {} # Need to deal with these after saving
-                for i, cell in enumerate(row):
-                    if header_row_field_names[i]: # skip blank
-                        if not import_log.is_empty(cell) or header_row_null_on_empty[i]:
-                            set_field_from_cell(import_log, new_object, header_row_field_names[i], cell)
-                        elif header_row_default[i]:
-                            set_field_from_cell(import_log, new_object, header_row_field_names[i], header_row_default[i])
-                new_object.save()
+                    elif import_log.import_type == "U":
+                        filters = {key_field_name: row[key_index]}
+                        new_object = model_class.objects.filter(**filters).first()
+                        if new_object == None:
+                            new_object = model_class()
+                            is_created = False
 
-                for i, cell in enumerate(row):
-                    if header_row_field_names[i]: # skip blank
-                        if not import_log.is_empty(cell) or header_row_null_on_empty[i]:
-                            set_method_from_cell(import_log, new_object, header_row_field_names[i], cell)
-                        elif header_row_default[i]:
-                            set_method_from_cell(import_log, new_object, header_row_field_names[i], header_row_default[i])
-                # set_custom_value() calls save() on its own, but the same cannot be assumed
-                # for other methods, e.g. set_password()
-                new_object.save()
+                    new_object.simple_import_m2ms = {} # Need to deal with these after saving
+                    for i, cell in enumerate(row):
+                        if header_row_field_names[i]: # skip blank
+                            if not import_log.is_empty(cell) or header_row_null_on_empty[i]:
+                                set_field_from_cell(import_log, new_object, header_row_field_names[i], cell)
+                            elif header_row_default[i]:
+                                set_field_from_cell(import_log, new_object, header_row_field_names[i], header_row_default[i])
+                    new_object.save()
 
-                for key in new_object.simple_import_m2ms.keys():
-                    value = new_object.simple_import_m2ms[key]
-                    m2m = getattr(new_object, key)
-                    m2m_model = type(m2m.model())
-                    related_field_name = RelationalMatch.objects.get(import_log=import_log, field_name=key).related_field_name
-                    m2m_object = m2m_model.objects.get(**{related_field_name:value})
-                    m2m.add(m2m_object)
+                    for i, cell in enumerate(row):
+                        if header_row_field_names[i]: # skip blank
+                            if not import_log.is_empty(cell) or header_row_null_on_empty[i]:
+                                set_method_from_cell(import_log, new_object, header_row_field_names[i], cell)
+                            elif header_row_default[i]:
+                                set_method_from_cell(import_log, new_object, header_row_field_names[i], header_row_default[i])
+                    # set_custom_value() calls save() on its own, but the same cannot be assumed
+                    # for other methods, e.g. set_password()
+                    new_object.save()
 
-                if is_created:
-                    LogEntry.objects.log_action(
-                        user_id         = request.user.pk,
-                        content_type_id = ContentType.objects.get_for_model(new_object).pk,
-                        object_id       = new_object.pk,
-                        object_repr     = smart_text(new_object),
-                        action_flag     = ADDITION
-                    )
-                    create_count += 1
-                else:
-                    LogEntry.objects.log_action(
-                        user_id         = request.user.pk,
-                        content_type_id = ContentType.objects.get_for_model(new_object).pk,
-                        object_id       = new_object.pk,
-                        object_repr     = smart_text(new_object),
-                        action_flag     = CHANGE
-                    )
-                    update_count += 1
-                ImportedObject.objects.create(
-                    import_log = import_log,
-                    object_id = new_object.pk,
-                    content_type = import_log.import_setting.content_type)
+                    for key in new_object.simple_import_m2ms.keys():
+                        value = new_object.simple_import_m2ms[key]
+                        m2m = getattr(new_object, key)
+                        m2m_model = type(m2m.model())
+                        related_field_name = RelationalMatch.objects.get(import_log=import_log, field_name=key).related_field_name
+                        m2m_object = m2m_model.objects.get(**{related_field_name:value})
+                        m2m.add(m2m_object)
+
+                    if is_created:
+                        LogEntry.objects.log_action(
+                            user_id         = request.user.pk,
+                            content_type_id = ContentType.objects.get_for_model(new_object).pk,
+                            object_id       = new_object.pk,
+                            object_repr     = smart_text(new_object),
+                            action_flag     = ADDITION
+                        )
+                        create_count += 1
+                    else:
+                        LogEntry.objects.log_action(
+                            user_id         = request.user.pk,
+                            content_type_id = ContentType.objects.get_for_model(new_object).pk,
+                            object_id       = new_object.pk,
+                            object_repr     = smart_text(new_object),
+                            action_flag     = CHANGE
+                        )
+                        update_count += 1
+                    ImportedObject.objects.create(
+                        import_log = import_log,
+                        object_id = new_object.pk,
+                        content_type = import_log.import_setting.content_type)
             except IntegrityError:
                 exc = sys.exc_info()
-                error_data += [row + ["Integrity Error", smart_text(exc[1][1])]]
+                error_data += [row + ["Integrity Error", smart_text(exc[1])]]
                 fail_count += 1
             except ObjectDoesNotExist:
                 exc = sys.exc_info()
@@ -433,13 +452,10 @@ def do_import(request, import_log_id):
                     error_data += [row + ["Value Error", smart_text(exc[1])]]
                 fail_count += 1
             except:
-                exc = sys.exc_info()
-                error_data += [row + ["Unknown Error", smart_text(exc[1])]]
+                error_data += [row + ["Unknown Error"]]
                 fail_count += 1
-        if commit:
-            transaction.commit()
-        else:
-            transaction.rollback()
+        if not commit:
+            transaction.savepoint_rollback(sid)
 
 
     if fail_count:
